package com.ruoyi.system.controller;

import org.springframework.web.bind.annotation.PostMapping;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.system.api.domain.SysFile;
import java.util.Arrays;
import com.ruoyi.common.core.utils.file.MimeTypeUtils;
import com.ruoyi.common.core.utils.file.FileTypeUtils;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import com.ruoyi.system.enums.BussLogEnum;
import org.springframework.web.bind.annotation.PutMapping;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.system.api.model.LoginUser;
import com.ruoyi.common.core.utils.StringUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.GetMapping;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.system.service.ISnServerService;
import com.ruoyi.system.service.ISysBussLogService;
import com.ruoyi.system.api.RemoteFileService;
import com.ruoyi.common.security.service.TokenService;
import org.springframework.beans.factory.annotation.Autowired;
import com.ruoyi.system.service.ISysUserService;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.core.web.controller.BaseController;

@RestController
@RequestMapping({ "/user/profile" })
public class SysProfileController extends BaseController
{
    @Autowired
    private ISysUserService userService;
    @Autowired
    private TokenService tokenService;
    @Autowired
    private RemoteFileService remoteFileService;
    @Autowired
    private ISysBussLogService sysBussLogService;
    @Autowired
    private ISnServerService snServerService;
    
    @GetMapping
    public AjaxResult profile() {
        final String username = SecurityUtils.getUsername();
        final SysUser user = this.userService.selectUserByUserName(username);
        final AjaxResult ajax = AjaxResult.success((Object)user);
        ajax.put("roleGroup", (Object)this.userService.selectUserRoleGroup(username));
        ajax.put("postGroup", (Object)this.userService.selectUserPostGroup(username));
        return ajax;
    }
    
    @Log(title = "个人信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult updateProfile(@RequestBody final SysUser user) {
        final LoginUser loginUser = SecurityUtils.getLoginUser();
        final SysUser currentUser = loginUser.getSysUser();
        currentUser.setNickName(user.getNickName());
        currentUser.setEmail(user.getEmail());
        currentUser.setPhonenumber(user.getPhonenumber());
        currentUser.setSex(user.getSex());
        if (StringUtils.isNotEmpty(user.getPhonenumber()) && !this.userService.checkPhoneUnique(currentUser)) {
            return this.error("修改用户'" + loginUser.getUsername() + "'失败，手机号码已存在");
        }
        if (StringUtils.isNotEmpty(user.getEmail()) && !this.userService.checkEmailUnique(currentUser)) {
            return this.error("修改用户'" + loginUser.getUsername() + "'失败，邮箱账号已存在");
        }
        if (this.userService.updateUserProfile(currentUser) > 0) {
            this.tokenService.setLoginUser(loginUser);
            return this.success();
        }
        return this.error("修改个人信息异常，请联系管理员");
    }
    
    @Log(title = "个人信息", businessType = BusinessType.UPDATE)
    @PutMapping({ "/updatePwd" })
    public AjaxResult updatePwd(final String oldPassword, String newPassword) {
        final String username = SecurityUtils.getUsername();
        final SysUser user = this.userService.selectUserByUserName(username);
        final String password = user.getPassword();
        if (!SecurityUtils.matchesPassword(oldPassword, password)) {
            return this.error("修改密码失败，旧密码错误");
        }
        if (SecurityUtils.matchesPassword(newPassword, password)) {
            return this.error("新密码不能与旧密码相同");
        }
        this.sysBussLogService.insertSysBussLog(BussLogEnum.EditPassword.getType(), user.getUserId(), user.getUserName(), "修改密码 " + oldPassword + " => " + newPassword);
        newPassword = SecurityUtils.encryptPassword(newPassword);
        if (this.userService.resetUserPwd(username, newPassword) > 0) {
            if (user.getUserType().equals("common")) {
                this.snServerService.logoutMerchant(user.getUserId());
            }
            final LoginUser loginUser = SecurityUtils.getLoginUser();
            loginUser.getSysUser().setPassword(newPassword);
            this.tokenService.setLoginUser(loginUser);
            return this.success();
        }
        return this.error("修改密码异常，请联系管理员");
    }
    
    @Log(title = "用户头像", businessType = BusinessType.UPDATE)
    @PostMapping({ "/avatar" })
    public AjaxResult avatar(@RequestParam("avatarfile") final MultipartFile file) {
        if (!file.isEmpty()) {
            final LoginUser loginUser = SecurityUtils.getLoginUser();
            final String extension = FileTypeUtils.getExtension(file);
            if (!StringUtils.equalsAnyIgnoreCase((CharSequence)extension, (CharSequence[])MimeTypeUtils.IMAGE_EXTENSION)) {
                return this.error("文件格式不正确，请上传" + Arrays.toString(MimeTypeUtils.IMAGE_EXTENSION) + "格式");
            }
            final R<SysFile> fileResult = (R<SysFile>)this.remoteFileService.upload(file);
            if (StringUtils.isNull((Object)fileResult) || StringUtils.isNull(fileResult.getData())) {
                return this.error("文件服务异常，请联系管理员");
            }
            final String url = ((SysFile)fileResult.getData()).getUrl();
            if (this.userService.updateUserAvatar(loginUser.getUsername(), url)) {
                final AjaxResult ajax = AjaxResult.success();
                ajax.put("imgUrl", (Object)url);
                loginUser.getSysUser().setAvatar(url);
                this.tokenService.setLoginUser(loginUser);
                return ajax;
            }
        }
        return this.error("上传图片异常，请联系管理员");
    }
}
